/*
	--- dat2octave.cc --- 

R. Schulte (roman.schulte89'at'gmail.com)
26.05.2014

This script was written for CinC-challenge 2014. It reads signal data from the specified *.dat file, searches for EC, BP and SV signals marking the related columns with numbers 1,2 or 3 in the first row of output matrix dx. Furthermore it produces *.qrs and *.wabp files containing annotators found by the functions wabp.c and gqrs.c

input:
- Name of the *.dat file to read.

output:
- Matrix dx for further computation in octave, containing all data red from the first ten minutes of the *.dat file.

- several *.qrs and *.wabp files written by the c-scripts wabp.c and grqs.c. The number of files depends on the number of ECG- and BP-signals included in the *.dat-file.

-------------
 * Changes:
 * 12.02.2015: If no ECG was found in the header (e.g. there was a "no signal" description as in record 42878), the first signal is defined as the ECG

*/

#include <octave/oct.h>
#include <stdio.h>
#include <malloc.h>
#include <wfdb/wfdb.h>
#include <stdlib.h>

DEFUN_DLD (dat2octave, args, , "bla")
{
	int i, j, nsig;
	WFDB_Sample *v;
	WFDB_Siginfo *s;
	std::string  tmp,dir,a,command;
	int fSamp = 0;
	int ECGfound = 0; // Check if an ECG signal was found in the header, if not, consider the first signal as the ECG (JWK)
	int BPfound = 0;  // Check if an BP  signal was found in the header, if not, consider the second signal as the BP (JWK)
	std::stringstream ss;


	charMatrix ch = args(0).char_matrix_value ();
  	tmp = ch.row_as_string (i);

        dir = ""+tmp;
	fSamp = sampfreq(&dir[0]);

	nsig = isigopen(&dir[0], NULL, 0);
	if (nsig < 1)
		exit(1);
	s = (WFDB_Siginfo *)malloc(nsig * sizeof(WFDB_Siginfo));
	if (isigopen(&tmp[0], s, nsig) != nsig)
		exit(1);
 	v = (WFDB_Sample *)malloc(nsig * sizeof(WFDB_Sample));

	Matrix dx (fSamp*600+1,nsig+1);

	int idxE = 1,idxB = 1;
	for (i=0; i<nsig; i++)
	{ 	a = s[i].desc;
		dx(0,i+1) = 0;
		ss.str("");		
		
		
		if ((a.find("ECG")!=std::string::npos) || (a.find("Ecg")!=std::string::npos) || (a.find("ecg")!=std::string::npos) || (a.find("Lead")!=std::string::npos) || (a.find("lead")!=std::string::npos) || (a.find("II")!=std::string::npos) || (a.find("III")!=std::string::npos) )
			{dx(0,i+1) = 1;
			ss << i;
			command = "gqrs -r " + dir +" -s "+ ss.str();
			system(command.c_str());
			ss.str("");
			ss << idxE;
			command = "rename 's/\.qrs$/.qrs"+ss.str()+"/' "+dir+".qrs";
			system(command.c_str());
			idxE += 1;
			ECGfound=1; //Found an ECG signal (JWK)
			}

		
		if ((a.find("BP")!=std::string::npos) || (a.find("PAP")!=std::string::npos) ||(a.find("ART")!=std::string::npos) ||(a.find("Pressure")!=std::string::npos))
			{dx(0,i+1) = 2;
			ss << i;
			command = "wabp -r " + dir +" -s "+ ss.str();
			system(command.c_str());
			ss.str("");
			ss << idxB;
			command = "rename 's/\.wabp$/.wabp"+ss.str()+"/' "+dir+".wabp";
			system(command.c_str());
			idxB += 1;
			BPfound=1; //Found an ECG signal (JWK)
			}

		if ((a.find("SV")!=std::string::npos) || (a.find("PLETH")!=std::string::npos))
			dx(0,i+1) = 3;
	}

	//If ECG was not found in header, then simply take the first signal as the ECG (JWK)
	if (ECGfound==0){
			i=0; //First signal == ECG if no other detected
			dx(0,i+1) = 1;
			ss << i;
			command = "gqrs -r " + dir +" -s "+ ss.str();
			system(command.c_str());
			ss.str("");
			ss << idxE;
			command = "rename 's/\.qrs$/.qrs"+ss.str()+"/' "+dir+".qrs";
			system(command.c_str());
			idxE += 1;}
	


	//If BP was not found in header, then simply take the second signal as the BP (JWK)
	if (BPfound==0 && nsig > 1){
			i=1; //Second signal == BP if no other detected
			dx(0,i+1) = 1;
			ss << i;
			command = "gqrs -r " + dir +" -s "+ ss.str();
			system(command.c_str());
			ss.str("");
			ss << idxE;
			command = "rename 's/\.qrs$/.qrs"+ss.str()+"/' "+dir+".qrs";
			system(command.c_str());
			idxE += 1;}



	for (i = 1; i < (s[0].nsamp+1); i++) 
	{
		if ((getvec(v) < 0)||(fSamp*600<i))
			break;
		for (j = 0; j < nsig; j++)
		{
			dx(i,0) = i;
			dx(i,j+1) = v[j];
		}	
	}
	dx(0,0) = fSamp;
	wfdbquit();
	return octave_value(dx);
}
